////////////////////////////////////////////////////////////////////////////////
//	Attachments.cpp:	Implements the Jet Attachments object class
//
//	This class defines the Attachment container class that holds all the attachments
//	for an Indexed Mesh resource file.  This is an optional chunk
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	NOTE: This is assumed to be an open-source project, so individual method
//	notes appear before each method in the CPP file rather than here in the
//	.h file.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Attachments.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CAttachments::CAttachments()
{
	strncpy (&m_theHeader.chATCHChunkID[0], "ATCH", 4);
	m_theHeader.dwVersion			= 100;
	m_theHeader.dwAttachmentCount	= 0;
}

CAttachments::~CAttachments()
{
//
//			Clean up the attachment objects pointed to by m_vAttachments
//
	ATTACHMENTVECTOR::iterator pAttachment = m_vAttachments.begin ();
	while (pAttachment != m_vAttachments.end ())
	{
		delete (CAttachment *) (*pAttachment);
		pAttachment++;
	}
	m_vAttachments.clear();
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this ATCH object in the final .IM file
//
//   COMMENTS:
//
//	This method will add up the total lengths of all its internal components,
//	and return the total lengths.
//
//    RETURN VALUES:
//		The full size of this object in the .IM file
//
DWORD CAttachments::GetFullSize()
{
	CAttachment *	ptrAttachment;
	DWORD			dwTotalSize	= 0;

	dwTotalSize += sizeof (m_theHeader);	// Header size
//
//			Add in the size of each attachment
//
	ATTACHMENTVECTOR::iterator pAttachment = m_vAttachments.begin ();
	while (pAttachment != m_vAttachments.end ())
	{
		ptrAttachment = (*pAttachment);
		dwTotalSize += ptrAttachment->GetFullSize();
		pAttachment++;
	}
//
//			Set the attachment count
//
	m_theHeader.dwAttachmentCount = m_vAttachments.size();
	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the reamining size of this ATCH object after its chunk size
//
//   COMMENTS:
//
//	This method will get the total length of this object in the .IM file, and
//	will subtract the leading "ATCH" text and chunk size DWORD to return the
//	remaining size of the object.
//
//    RETURN VALUES:
//		The remaining size of this object after the chunk size
//
DWORD CAttachments::GetRemainingSize()
{
	m_theHeader.dwChunkSize = GetFullSize () - (4 + sizeof (DWORD));
	return m_theHeader.dwChunkSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this object and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the material object, which includes
//	attachments, to the output *.IM Indexed Mesh file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CAttachments::WriteToOutputFile(HANDLE	hOutputFile)
{
//
//			Write out the ATCH header (with its fixed fields)
//
	GetRemainingSize();				// fill in the size value
	if (! PerformTheWrite(hOutputFile, &m_theHeader, sizeof (m_theHeader), 
						  "Write of ATCH Header failed"))
	{
		return false;
	}
//
//			Write out each attachment
//
	CAttachment *	ptrAttachment;
	ATTACHMENTVECTOR::iterator pAttachment = m_vAttachments.begin ();
	while (pAttachment != m_vAttachments.end ())
	{
		ptrAttachment = (*pAttachment);
		ptrAttachment->WriteToOutputFile (hOutputFile);
		pAttachment++;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CAttachment object and put it into the attachment vector
//
//   COMMENTS:
//
//	This method will create a new CAttachment object and put it into the attachment vector.
//	It will then return a pointer to that CAttachment object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentAttachment	Returned pointer to the new CAttachment object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CAttachments::CreateNewAttachment (CAttachment*& pCurrentAttachment)
{
//			Create the attachment, insure that it is valid, and put it into the 
//			attachment vector
//
	pCurrentAttachment = new CAttachment ();
	pCurrentAttachment->AssertValid();
	m_vAttachments.push_back (pCurrentAttachment);
	return true;
}


////////////////////////////////////////////////////////////////////////////////
