////////////////////////////////////////////////////////////////////////////////
//	BaseResourceFormatter.cpp:	Implements the class for Jet resource file formatting
//
//	This is the base class containing all of the resource formatting classes
//	needed to create Jet resource input files.  As of this writing, only the
//	*.IM file formatting routines exist.  Some additional work will be needed
//	to implement the other Jet resource file formats.
//
//	This base class contains all the IMObject classes used to represent headers,
//	chunks, or other data elements of the *.IM file.  It controls their creation
//	provides methots to fill them with data, and the executes the utlimate task
//	of writing their data to the final output file.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BaseResourceFormatter.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CBaseResourceFormatter::CBaseResourceFormatter()
{
//
//			Load the addresses of related objects
//
	m_IMHeader.SetMainInfoHdrAddr(&m_InfoHeader);
	m_InfoHeader.SetMainMeshVectorAddr (&m_vMesh);
	m_IMHeader.SetInfluenceObjectAddr (&m_Influence);
	m_IMHeader.SetAttachmentsObjectAddr (&m_Attachments);
//
//			Create the animation file objects
//
	m_pKINHeader = new CKINHeader ();
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CBaseResourceFormatter::~CBaseResourceFormatter()
{
	Cleanup ();
	delete m_pKINHeader;	// on destruction, we don't need a new one of these.
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Sets the value of the Indexed Mesh output file handle
//
//   COMMENTS:
//
//	This method sets the value of the IM output file handle to use to write out the
//	Jet Indexec Mesh resource file being created.  It is up to the DataSource object
//	to determine what and where the output file should be, and to open and close the
//	file as needed.  This class simply uses that file.
//
//    RETURN VALUES:
//		NONE
//
void CBaseResourceFormatter::SetIMOutputFileHandle(HANDLE hOutputFile)
{
	m_hIMOutputFile = hOutputFile;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Sets the value of the Animatin (KIN) output file handle
//
//   COMMENTS:
//
//	This method sets the value of the KIN output file handle to use to write out the
//	Jet animation resource file being created.  It is up to the DataSource object to
//	determine what and where the output file should be, and to open and close the
//	file as needed.  This class simply uses that file.
//
//    RETURN VALUES:
//		NONE
//
void CBaseResourceFormatter::SetKINOutputFileHandle(HANDLE hOutputFile)
{
	m_hKINOutputFile = hOutputFile;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CMesh object and put it into the mesh vector
//
//   COMMENTS:
//
//	This method will create a new CMesh object and put it into the mesh vector.
//	It will then return a pointer to that CMesh object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentMesh		Returned pointer to the new CMesh object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CBaseResourceFormatter::CreateNewMesh(CMesh*& pCurrentMesh)
{
//			Create the mesh, insure that it is valid, and put it into the 
//			mesh vector
//
	int	nMeshIndex = m_vMesh.size();
	pCurrentMesh = new CMesh ();
	pCurrentMesh->AssertValid();
	m_vMesh.push_back (pCurrentMesh);
	pCurrentMesh->SetChunkIndex (m_vMesh.size() - 1);
	if (m_vMesh.size() <= nMeshIndex)
		return false;
	else
		return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Set the model position in the information header
//
//   COMMENTS:
//
//	This method will set the X, Y, and Z position values for the INFO header
//
//	  X  The X axis position of the model
//	  Y  The Y axis position of the model
//	  Z  The Z axis position of the model
//
//    RETURN VALUES:
//		NONE
//
void CBaseResourceFormatter::SetModelPosition (float X, float Y, float Z)
{
	m_InfoHeader.SetPosition (X, Y, Z);
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Writes out the final .IM file to load 3D models into Jet.
//
//   COMMENTS:
//
//	This method will write out all the components of the Jet resource file that
//	are now in its internal objects.
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CBaseResourceFormatter::WriteOutTheIMFile()
{
	m_IMHeader.GetRemainingSize ();
	m_IMHeader.WriteToOutputFile (m_hIMOutputFile);
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Writes out the final KIN file to load 3D models into Jet.
//
//   COMMENTS:
//
//	This method will write out all the components of the Jet resource file that
//	are now in its internal objects.
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CBaseResourceFormatter::WriteOutTheKINFile ()
{
	m_pKINHeader->GetRemainingSize ();
	m_pKINHeader->WriteToOutputFile (m_hKINOutputFile);
	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Clean up allocated memory in Jet components
//
//   COMMENTS:
//
//	This method will clean up the memory allocated for mesh, vector, face, and all
//	other Jet objects.  In many cases, this just means deleting the base objects
//	as the destructors of each object should clean up their own internal structures
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CBaseResourceFormatter::Cleanup ()
{
//
//			Clean up the mesh objects pointed to by m_vMesh
//
	MESHVECTOR::iterator pMesh = m_vMesh.begin ();
	while (pMesh != m_vMesh.end ())
	{
		delete (CMesh *) (*pMesh);
		pMesh++;
	}
	m_vMesh.clear();
	return true;
//
//			Replace the animation file classes with a fresh set
//
	delete m_pKINHeader;
	m_pKINHeader = new CKINHeader ();

}



////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Return a pointer to the mesh indicated by the given name
//
//   COMMENTS:
//
//	This method will return the pointer to the indicatec mesh if it exists in 
//	the mesh vector, or will return zero indicating that the mesh does not exist.
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CBaseResourceFormatter::GetMesh(CMesh*&	pCurrentMesh,
									 char *		szMeshName)
{
//
//			Look for the indicated mesh in the mesh vector
//
	CMesh *	ptrMesh;

	MESHVECTOR::iterator pMesh = m_vMesh.begin ();
	while (pMesh != m_vMesh.end ())
	{
		ptrMesh = (*pMesh);
		if (stricmp (ptrMesh->GetMeshName(), szMeshName) == EQUAL)
		{
			pCurrentMesh = (*pMesh);
			return true;
		}
		pMesh++;
	}
	return false;
}

////////////////////////////////////////////////////////////////////////////////

