////////////////////////////////////////////////////////////////////////////////
//	Bone.cpp:	Implementation of the Jet bone object class
//
//	This class defines individual bones within the influence chuck.  It is not
//	itself a complete chunk, but occurs zero to multiple times within each
//	normal Influence chunk.
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Bone.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CBone::CBone()
{

}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CBone::~CBone()
{
//
//			Clean up the bone influence objects pointed to by m_vBoneInfluence
//
	BONEINFLUENCEVECTOR::iterator pInfluence = m_vBoneInfluence.begin ();
	while (pInfluence != m_vBoneInfluence.end ())
	{
		delete (CBoneInfluenceChunk *) (*pInfluence);
		pInfluence++;
	}
	m_vBoneInfluence.clear();
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this object in the final .IM file
//
//   COMMENTS:
//
//	This method will add up the total lengths of all its internal components,
//	and return the total lengths.
//
//    RETURN VALUES:
//		The full size of this object in the .IM file
//
DWORD CBone::GetFullSize ()
{
	DWORD dwTotalSize = 0;
//
//			Start with the attribute values
//
	dwTotalSize += (2 * sizeof (DWORD));		// string size lengths
	dwTotalSize += m_dwBoneNameSize;			// bone name length
	dwTotalSize += m_dwParentNameSize;			// parent bone name length
	dwTotalSize += sizeof (m_theValues);		// remaining attribute lengths
//
//			Add in the size of all the contained bone influence objects
//
	CBoneInfluenceChunk * ptrInfluence;
	BONEINFLUENCEVECTOR::iterator pInfluence = m_vBoneInfluence.begin ();
	while (pInfluence != m_vBoneInfluence.end ())
	{
		ptrInfluence = (*pInfluence);
		dwTotalSize += ptrInfluence->GetFullSize();
		pInfluence++;
	}

	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the reamining size of this  object after its chunk size
//
//   COMMENTS:
//
//	This object has no length field within a standard header, so it will simply
//	return the full length for the remaining length
//
//    RETURN VALUES:
//		The remaining size of this object
//
DWORD CBone::GetRemainingSize ()
{
	return GetFullSize ();
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this object and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the influence vertes object to the
//	output *.IM Indexed Mesh file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CBone::WriteToOutputFile (HANDLE	hOutputFile)
{
//
//			Write out the length of the bone name
//
	if (! PerformTheWrite(hOutputFile, &m_dwBoneNameSize, sizeof (m_dwBoneNameSize), 
						  "Write of bone name size failed"))
	{
		return false;
	}
//
//			Write out the name padded to the next DWORD boundary
//
	if (! PerformTheWrite(hOutputFile, &m_strBoneName, m_dwBoneNameSize, 
						  "Write of bone name failed"))
	{
		return false;
	}
//
//			Write out the length of the parent bone name
//
	if (! PerformTheWrite(hOutputFile, &m_dwParentNameSize, sizeof (m_dwParentNameSize), 
						  "Write of bone name size failed"))
	{
		return false;
	}
//
//			Write out the name padded to the next DWORD boundary
//
	if (! PerformTheWrite(hOutputFile, &m_strParentName, m_dwParentNameSize, 
						  "Write of bone name failed"))
	{
		return false;
	}
//
//			Write out the rest of the bone "header" data
//
	m_theValues.dwCount = m_vBoneInfluence.size();
	if (! PerformTheWrite(hOutputFile, &m_theValues, sizeof(m_theValues), 
						  "Write of bone name failed"))
	{
		return false;
	}
//
//			Write out each bone influence
//
	CBoneInfluenceChunk * ptrInfluence;
	BONEINFLUENCEVECTOR::iterator pInfluence = m_vBoneInfluence.begin ();
	while (pInfluence != m_vBoneInfluence.end ())
	{
		ptrInfluence = (*pInfluence);
		ptrInfluence->WriteToOutputFile (hOutputFile);
		pInfluence++;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CBoneInfluenceChunk object and put it into the bone
//			  influence vector
//
//   COMMENTS:
//
//	This method will create a new CBoneInfluenceChunk object and put it into the bone influence vector.
//	It will then return a pointer to that CBoneInfluenceChunk object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentInfluence	Returned pointer to the new CBoneInfluenceChunk object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CBone::CreateNewInfluence(CBoneInfluenceChunk*& pCurrentInfluence)
{
//			Create the bone influence, insure that it is valid, and put it into the 
//			bone influence vector
//
	pCurrentInfluence = new CBoneInfluenceChunk ();
	pCurrentInfluence->AssertValid();
	pCurrentInfluence->SetIndex (m_vBoneInfluence.size());
	m_vBoneInfluence.push_back (pCurrentInfluence);
	return true;
}


////////////////////////////////////////////////////////////////////////////////
