////////////////////////////////////////////////////////////////////////////////
//	Face.cpp:	The Jet Face object class implementation
//
//	This class implements the Face object used to describe an individual
//	triangle in a mesh.  It contains indices into the vertex array to indicate the
//	three defining vertices, a normal vector, and other values that define each
//	individual face.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Face.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CFace::CFace()
{
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CFace::~CFace()
{
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Calculate the normal vector from three passed vertex objects
//
//   COMMENTS:
//
//	This method will use the X, Y, Z coordinates of the three passed vertex
//	objects to calculate two triangle edge vectors.  It will then use the cross
//	product of those vectors to calculate the vector for the normal.  Finally,
//	it will "normalize the normal" to create a normal value with a unit length.
//
//	NOTE:  The passed vertex objects are assumed to be in "counter-clockwize"
//	order indicating a right-handed system.
//
//	  pV1			Pointer to the first vertex for this face
//	  pV2			Pointer to the second vertex for this face
//	  pV3			Pointer to the third vertex for this face
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CFace::CalculateNormal(CVertex * pV1,
							CVertex * pV2,
							CVertex * pV3)
{
	float	afVectorA [3];
	float	afVectorB [3];
	float	afXProduct[3];
	float	fXProductLength;
	int		x = 0;
	int		y = 1;
	int		z = 2;
//
//			Calculate the two edge vectors from the vertexes
//
	afVectorA[x] = pV2->GetCoords(x) - pV3->GetCoords(x);
	afVectorA[y] = pV2->GetCoords(y) - pV3->GetCoords(y);
	afVectorA[z] = pV2->GetCoords(z) - pV3->GetCoords(z);

	afVectorB[x] = pV2->GetCoords(x) - pV1->GetCoords(x);
	afVectorB[y] = pV2->GetCoords(y) - pV1->GetCoords(y);
	afVectorB[z] = pV2->GetCoords(z) - pV1->GetCoords(z);
//
//			Calculate the cross product
//
	afXProduct[x] = afVectorA[y] * afVectorB[z] - afVectorA[z] * afVectorB[y];
	afXProduct[y] = afVectorA[z] * afVectorB[x] - afVectorA[x] * afVectorB[z];
	afXProduct[z] = afVectorA[x] * afVectorB[y] - afVectorA[y] * afVectorB[x];
//
//			Normalize the cross product
//
	fXProductLength = (float) sqrt ((afXProduct[x] * afXProduct[x]) + 
									(afXProduct[y] * afXProduct[y]) + 
									(afXProduct[z] * afXProduct[z]));

	m_afNormal[x] = afXProduct[x] / fXProductLength;
	m_afNormal[y] = afXProduct[y] / fXProductLength;
	m_afNormal[z] = afXProduct[z] / fXProductLength;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write the vertex indices to the output file
//
//   COMMENTS:
//
//	This method will write the vertex indices for this triangle to the output
//	file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CFace::WriteVertsToOutputFile (HANDLE hOutputFile)
{
	if (! PerformTheWrite(hOutputFile, &m_anVertexIndices, sizeof (m_anVertexIndices), 
						  "Write of face vertex indices failed"))
	{
		return false;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write the face normals to the output file
//
//   COMMENTS:
//
//	This method will write the normal values for this face to the output file.
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CFace::WriteNormalsToOutputFile(HANDLE hOutputFile)
{
	if (! PerformTheWrite(hOutputFile, &m_afNormal, sizeof (m_afNormal), 
						  "Write of face normals failed"))
	{
		return false;
	}
	return true;
}
////////////////////////////////////////////////////////////////////////////////
