////////////////////////////////////////////////////////////////////////////////
//	Frame.cpp:	Implement the Jet CFrame class
//
//	This class defines one frame within an animation (kin) file.
//
//	This class the position and rotation for every bone in the skeleton defined
//	within the animation file.  There will be one CFrame object for each frame
//	identified in the CKINInfoHeader object's dwFrameCount value
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Frame.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CFrame::CFrame()
{
	strncpy (&m_theHeader.chChunkID[0], "FRAM", 4);
	m_theHeader.dwChunkSize		= 0;
	m_theHeader.dwVersion		= 100;
	m_theHeader.dwFrameNo		= 0;
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CFrame::~CFrame()
{
//
//			Clean up the frame detail objects pointed to by m_vFrameDetails
//
	FRAMEDETAILVECTOR::iterator pFrameDetails = m_vFrameDetails.begin ();
	while (pFrameDetails != m_vFrameDetails.end ())
	{
		delete (CFrameDetail *) (*pFrameDetails);
		pFrameDetails++;
	}
	m_vFrameDetails.clear();
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this CFrame object in the final .KIN file
//
//   COMMENTS:
//
//	This method will return the length of the full CFrame heading..
//
//    RETURN VALUES:
//		The full size of this object in the .KIN file
//
DWORD CFrame::GetFullSize()
{
	CFrameDetail *	ptrFrameDetail;
	DWORD			dwTotalSize	= sizeof (m_theHeader);	// start with the header size
//
//			Count the frame detail objects we will use, and sum their total size.
//
	FRAMEDETAILVECTOR::iterator pFrameDetail = m_vFrameDetails.begin ();
	while (pFrameDetail != m_vFrameDetails.end ())
	{
		ptrFrameDetail = (*pFrameDetail);
		dwTotalSize += ptrFrameDetail->GetFullSize ();
		pFrameDetail++;
	}

	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the reamining size of this FRAM object after its chunk size
//
//   COMMENTS:
//
//	This method will get the total length of this object in the .KIN file, and
//	will subtract the leading "EVNT" text and chunk size DWORD to return the
//	remaining size of the object.
//
//    RETURN VALUES:
//		The remaining size of this object after the chunk size
//
DWORD CFrame::GetRemainingSize()
{
	m_theHeader.dwChunkSize = GetFullSize () - (4 + sizeof (DWORD));
	return m_theHeader.dwChunkSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this header and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the FRAM Header, which happens to be
//	the whole file, to the output *.KIN Animation file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CFrame::WriteToOutputFile(HANDLE			hOutputFile)
{
	CFrameDetail *	ptrFrameDetail;
//
//			Write out the CFrame header
//
	GetRemainingSize();
	if (! PerformTheWrite(hOutputFile, &m_theHeader, sizeof (m_theHeader), 
						  "Write of CFrameDetail failed"))
	{
		return false;
	}
//
//			Write out each CFrame object
//
	FRAMEDETAILVECTOR::iterator pFrameDetail = m_vFrameDetails.begin ();
	while (pFrameDetail != m_vFrameDetails.end ())
	{
		ptrFrameDetail = (*pFrameDetail);
		ptrFrameDetail->WriteToOutputFile (hOutputFile);
		pFrameDetail++;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CFrameDetail object and put it into the FrameDetail vector
//
//   COMMENTS:
//
//	This method will create a new CFrameDetail object and put it into the FrameDetail vector.
//	It will then return a pointer to that CFrameDetail object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentFrameDetail		Returned pointer to the new CFrameDetail object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CFrame::CreateNewFrameDetail(CFrameDetail*& pCurrentFrameDetail)
{
//			Create the FrameDetail, insure that it is valid, and put it into the 
//			FrameDetail vector
//
	int	nFrameDetailIndex = m_vFrameDetails.size();
	pCurrentFrameDetail = new CFrameDetail ();
	pCurrentFrameDetail->AssertValid();
	m_vFrameDetails.push_back (pCurrentFrameDetail);
	if (m_vFrameDetails.size() <= nFrameDetailIndex)
		return false;
	else
		return true;
}

