////////////////////////////////////////////////////////////////////////////////
//	IMHeader.cpp:	The Jet IM File Header class
//
//	This class implements the IM File Header, which is the root class for everything
//	else in the file.  Specifically, the File Header contains the INFO Header
//	which in turns contains all the meshes, the influence chunk, etc.
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Note that the length in this case is the length of the entire file, not just
//	the length of the header.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "IMHeader.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CIMHeader::CIMHeader()
{
	strncpy (&m_theHeader.chJetFormatID[0], "JIRF", 4);
	m_theHeader.dwFileSize = 0;
	strncpy (&m_theHeader.chIMFormatID[0],  "IDXM", 4);
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CIMHeader::~CIMHeader()
{

}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this IM object in the final .IM file
//
//   COMMENTS:
//
//	This method will return the length of the full INFO heading.  Note,
//	however, that as the room of the chain of objects in the file, this
//	routine should never be called by an object further up the chain.
//
//    RETURN VALUES:
//		The full size of this object in the .IM file
//
DWORD CIMHeader::GetFullSize()
{
	return sizeof (m_theHeader);
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the size of this file remaining after the size file
//
//   COMMENTS:
//
//	This method will interigate all objects that will be "contained with it",
//	and will calculate the total size of all such objects plus its own size
//	after the initial size DWORD value
//
//    RETURN VALUES:
//		The remaining length of the entire .IM file after the length field in
//		this object.
//
DWORD CIMHeader::GetRemainingSize()
{
	DWORD dwTotalSize = 0;
//
//			Get the remaining size of this IM Header chunk
//
	dwTotalSize += sizeof (m_theHeader) - (4 + sizeof (DWORD));
//
//			Add the full size of the INFO Header chunk which includes
//			all the included meshes in the file.
//
	dwTotalSize += m_pInfoHeader->GetFullSize ();
//
//			Add the influences (which are listed as optional but are always present
//
	dwTotalSize += m_pInfluence->GetFullSize ();
//
//			Add the attachments (which are in fact optional)
//
	if (m_pAttachments->GetAttachmentCount() > 0)
		dwTotalSize += m_pAttachments->GetFullSize ();


	m_theHeader.dwFileSize = dwTotalSize;
	return m_theHeader.dwFileSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this header and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the IM Header, which happens to be
//	the whole file, to the output *.IM Indexed Mesh file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CIMHeader::WriteToOutputFile (HANDLE hOutputFile)
{
	if (! PerformTheWrite(hOutputFile, &m_theHeader, sizeof (m_theHeader), 
						  "Write of IMHeader failed"))
	{
		return false;
	}
//
//			Write the INFO header and all its data
//
	if (! m_pInfoHeader->WriteToOutputFile (hOutputFile))
	{
		return false;
	}
//
//			Write the Influence header and all its data
//
	if (! m_pInfluence->WriteToOutputFile (hOutputFile))
	{
		return false;
	}
//
//			Write the Attachements header and all its data if there is any
//
	if (m_pAttachments->GetAttachmentCount() > 0)
		if (! m_pAttachments->WriteToOutputFile (hOutputFile))
		{
			return false;
		}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Set the address of the INFO header object 
//
//   COMMENTS:
//
//	This method will set the address of the INFO header object which is used
//	to hold all the mesobjects hes that this IM header must handle.
//
//	  pInfoHeader		The address to use
//
//    RETURN VALUES:
//		None.
//
void CIMHeader::SetMainInfoHdrAddr(CInfoHeader *pInfoHeader)
{
	m_pInfoHeader = pInfoHeader;
}
////////////////////////////////////////////////////////////////////////////////
