////////////////////////////////////////////////////////////////////////////////
//	Influence.cpp:	The Jet Influence object class
//
//	This class implements the Influence object used to describe the influences
//	between meshes, bones, etc. in an articulated model.
//
//	Auran claims this is an optional chunk for simple models non-articulated
//	models, but in fact Jet will reject the IM file if this chunk is not
//	present, even it is indicates that there are no bones present.
//
//	This initial implementation of this class only serves to complete simple
//	non-articulated models, and may not be sufficient for truly articulated
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Influence.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CInfluence::CInfluence()
{
	strncpy (&m_theHeader.chInfluenceChunkID[0], "INFL", 4);
	m_theHeader.dwChunkSize	= GetRemainingSize ();
	m_theHeader.dwVersion	= 100;
	m_theHeader.dwBoneCount	= 0;
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CInfluence::~CInfluence()
{

}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this INFL object in the final .IM file
//
//   COMMENTS:
//
//	This method will return the length of the full INFL heading..
//
//    RETURN VALUES:
//		The full size of this object in the .IM file
//
DWORD CInfluence::GetFullSize()
{
	DWORD	dwTotalSize	= 0;
//
//			Start with the size of the header
//
	dwTotalSize += sizeof (m_theHeader);
//
//			Add in the size of all the contained bone objects
//
	CBone * ptrBone;
	BONEVECTOR::iterator pBone = m_vBone.begin ();
	while (pBone != m_vBone.end ())
	{
		ptrBone = (*pBone);
		dwTotalSize += ptrBone->GetFullSize();
		pBone++;
	}
	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the reamining size of this INFL object after its chunk size
//
//   COMMENTS:
//
//	This method will get the total length of this object in the .IM file, and
//	will subtract the leading "INFL" text and chunk size DWORD to return the
//	remaining size of the object.
//
//    RETURN VALUES:
//		The remaining size of this object after the chunk size
//
DWORD CInfluence::GetRemainingSize()
{
	m_theHeader.dwChunkSize = GetFullSize ()   - (4 + sizeof (DWORD));
	return m_theHeader.dwChunkSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this header and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the INFL Header to the output *.IM
//	Indexed Mesh file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CInfluence::WriteToOutputFile(HANDLE hOutputFile)
{
//
//			Make sure the bone count is set, and then
//			Write out the INFO header
//
	GetRemainingSize ();
	m_theHeader.dwBoneCount = m_vBone.size();
	if (! PerformTheWrite(hOutputFile, &m_theHeader, sizeof (m_theHeader), 
						  "Write of INFL Header failed"))
	{
		return false;
	}
//
//			Write out each bone
//
	CBone * ptrBone;
	BONEVECTOR::iterator pBone = m_vBone.begin ();
	while (pBone != m_vBone.end ())
	{
		ptrBone = (*pBone);
		ptrBone->WriteToOutputFile (hOutputFile);
		pBone++;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CBone object and put it into the bone vector
//
//   COMMENTS:
//
//	This method will create a new CBone object and put it into the bone vector.
//	It will then return a pointer to that CBone object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentBone	Returned pointer to the new CBone object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CInfluence::CreateNewBone(CBone*& pCurrentBone)
{
//			Create the bone, insure that it is valid, and put it into the 
//			bone vector
//
	pCurrentBone = new CBone ();
	pCurrentBone->AssertValid();
	m_vBone.push_back (pCurrentBone);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
