////////////////////////////////////////////////////////////////////////////////
//	KINHeader.cpp:	The Jet KIN File Header class
//
//	This class defines the KIN File Header, which is the root class for everything
//	else in the file.  Specifically, the File Header contains the INFO Header
//	which in turns contains all the information on bones and their movements, etc.
//
//	Like most of the Jet Objects, it is derived from CBaseINObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Note that the length in this case is the length of the entire file, not just
//	the length of the header.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "KINHeader.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CKINHeader::CKINHeader()
{
	strncpy (&m_theHeader.chJetFormatID[0], "JIRF", 4);
	m_theHeader.dwFileSize = 0;
	strncpy (&m_theHeader.chKINFormatID[0],  "ANIM", 4);

	m_theEvent.SetMainEventVectorAddr (&m_vEvents);			// set the event vector's address
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CKINHeader::~CKINHeader()
{
//
//			Clean up the event objects pointed to by m_vEvents
//
	EVENTVECTOR::iterator pEvent = m_vEvents.begin ();
	while (pEvent != m_vEvents.end ())
	{
		delete (CEvent *) (*pEvent);
		pEvent++;
	}
	m_vEvents.clear();

//
//			Clean up the frame objects pointed to by m_vFrames
//
	FRAMEVECTOR::iterator pFrame = m_vFrames.begin ();
	while (pFrame != m_vFrames.end ())
	{
		delete (CFrame *) (*pFrame);
		pFrame++;
	}
	m_vFrames.clear();

}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this KIN object in the final .KIN file
//
//   COMMENTS:
//
//	This method will return the length of the full INFO heading.  Note,
//	however, that as the room of the chain of objects in the file, this
//	routine should never be called by an object further up the chain.
//
//    RETURN VALUES:
//		The full size of this object in the .KIN file
//
DWORD CKINHeader::GetFullSize()
{
	DWORD dwTotalSize = 0;
//
//			Get the size of this KIN Header chunk
//
	dwTotalSize += sizeof (m_theHeader);
//
//			Add the full size of the INFO Header chunk.
//
	dwTotalSize += m_KINInfoHeader.GetFullSize ();
//
//			Add the full size of the Event header chunk
//
	dwTotalSize += m_theEvent.GetFullSize ();
//
//			Add the full size of the skeleton chunk
//
	dwTotalSize += m_theSkeleton.GetFullSize ();
//
//			Count the frame objects we will use, and sum their total size.
//
	CFrame *	ptrFrame;

	m_KINInfoHeader.SetFrameCount (0);
	FRAMEVECTOR::iterator pFrame = m_vFrames.begin ();
	while (pFrame != m_vFrames.end ())
	{
		ptrFrame = (*pFrame);
		dwTotalSize += ptrFrame->GetFullSize ();
		m_KINInfoHeader.SetFrameCount (m_KINInfoHeader.GetFrameCount () + 1);
		pFrame++;
	}

	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the size of this file remaining after the size file
//
//   COMMENTS:
//
//	This method will interigate all objects that will be "contained with it",
//	and will calculate the total size of all such objects plus its own size
//	after the initial size DWORD value
//
//    RETURN VALUES:
//		The remaining length of the entire .KIN file after the length field in
//		this object.
//
DWORD CKINHeader::GetRemainingSize()
{
	m_theHeader.dwFileSize = GetFullSize () - (4 + sizeof (DWORD));
	return m_theHeader.dwFileSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this header and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the KIN Header, which happens to be
//	the whole file, to the output *.KIN animation file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CKINHeader::WriteToOutputFile (HANDLE hOutputFile)
{
	if (! PerformTheWrite(hOutputFile, &m_theHeader, sizeof (m_theHeader), 
						  "Write of KINHeader failed"))
	{
		return false;
	}
//
//			Write the INFO header and all its data
//
	if (! m_KINInfoHeader.WriteToOutputFile (hOutputFile))
	{
		return false;
	}
//
//			Write the EVNT header and all its data
//
	if (! m_theEvent.WriteToOutputFile (hOutputFile))
	{
		return false;
	}
//
//			Write the SKEL header and all its data
//
	if (! m_theSkeleton.WriteToOutputFile (hOutputFile))
	{
		return false;
	}
//
//			Write out the frame data
//
	CFrame *	ptrFrame;

	FRAMEVECTOR::iterator pFrame = m_vFrames.begin ();
	while (pFrame != m_vFrames.end ())
	{
		ptrFrame = (*pFrame);
		ptrFrame->WriteToOutputFile (hOutputFile);
		pFrame++;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CEventDetail object and put it into the EventDetail vector
//
//   COMMENTS:
//
//	This method will create a new CEventDetail object and put it into the EventDetail vector.
//	It will then return a pointer to that CEventDetail object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentEventDetail		Returned pointer to the new CEventDetail object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CKINHeader::CreateNewEventDetail(CEventDetail*& pCurrentEventDetail)
{
//			Create the EventDetail, insure that it is valid, and put it into the 
//			EventDetail vector
//
	int	nEventDetailIndex = m_vEvents.size();
	pCurrentEventDetail = new CEventDetail ();
	pCurrentEventDetail->AssertValid();
	m_vEvents.push_back (pCurrentEventDetail);
	if (m_vEvents.size() <= nEventDetailIndex)
		return false;
	else
		return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CFrame object and put it into the Frames vector
//
//   COMMENTS:
//
//	This method will create a new CFrame object and put it into the Frames vector.
//	It will then return a pointer to that CFrame object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentFrame		Returned pointer to the new CFrame object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CKINHeader::CreateNewFrame(CFrame*& pCurrentFrame)
{
//			Create the Frame, insure that it is valid, and put it into the 
//			Frames vector
//
	int	nFrameIndex = m_vFrames.size();
	pCurrentFrame = new CFrame ();
	pCurrentFrame->AssertValid();
	m_vFrames.push_back (pCurrentFrame);
	if (m_vFrames.size() <= nFrameIndex)
		return false;
	else
		return true;
}


////////////////////////////////////////////////////////////////////////////////
