////////////////////////////////////////////////////////////////////////////////
//	Material.cpp:	The Jet Material object class
//
//	This class implements the Material object used to describe the material
//	used on a specific mesh.  A mesh can contain one material, but a material
//	can contain multiple textures.
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Material.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
//	Load the base values with defaults.
//
CMaterial::CMaterial()
{
	strncpy (&m_theHeader.chMATLChunkID[0], "MATL", 4);
	m_theHeader.dwVersion			= 102;
	theAfterHeader.dwNumProperties	= 0;	// default to no additional properties (not yet supported)
	theAfterHeader.dwTwoSided		= 0;	// default to no doing two-sided
	theAfterHeader.fOpacity			= 1.0;	// default to completely opaque
	theAfterHeader.afAmbientRGB [0] = 1.0;	// deafult most items to white
	theAfterHeader.afAmbientRGB [1] = 1.0;
	theAfterHeader.afAmbientRGB [2] = 1.0;
	theAfterHeader.afDiffuseRGB [0] = 1.0;
	theAfterHeader.afDiffuseRGB [1] = 1.0;
	theAfterHeader.afDiffuseRGB [2] = 1.0;
	theAfterHeader.afSpecularRGB[0] = 1.0;
	theAfterHeader.afSpecularRGB[1] = 1.0;
	theAfterHeader.afSpecularRGB[2] = 1.0;
	theAfterHeader.afEmissiveRGB[0] = 0.0;	// default emissiv to none
	theAfterHeader.afEmissiveRGB[1] = 0.0;
	theAfterHeader.afEmissiveRGB[2] = 0.0;
	theAfterHeader.fShininess		= 0.0;
	theAfterHeader.dwTextureCount	= 0;
	m_pImageTexture = NULL;
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CMaterial::~CMaterial()
{
//
//			Clean up the texture objects pointed to by m_vTexture
//
	TEXTUREVECTOR::iterator pTexture = m_vTexture.begin ();
	while (pTexture != m_vTexture.end ())
	{
		delete (CTexture *) (*pTexture);
		pTexture++;
	}
	m_vTexture.clear();
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this MATL object in the final .IM file
//
//   COMMENTS:
//
//	This method will add up the total lengths of all its internal components,
//	and return the total lengths.
//
//    RETURN VALUES:
//		The full size of this object in the .IM file
//
DWORD CMaterial::GetFullSize()
{
	CTexture *	ptrTexture;
	DWORD		dwTotalSize	= 0;

	dwTotalSize = sizeof (m_theHeader) +	// Header size
				  GetNameSize (m_strDecoratedName) +	// material name size
				  sizeof (theAfterHeader);	// rest of the header size
//
//			Add in the size of each texture
//
	TEXTUREVECTOR::iterator pTexture = m_vTexture.begin ();
	while (pTexture != m_vTexture.end ())
	{
		ptrTexture = (*pTexture);
		dwTotalSize += ptrTexture->GetFullSize();
		pTexture++;
	}

	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the reamining size of this MATL object after its chunk size
//
//   COMMENTS:
//
//	This method will get the total length of this object in the .IM file, and
//	will subtract the leading "MATL" text and chunk size DWORD to return the
//	remaining size of the object.
//
//    RETURN VALUES:
//		The remaining size of this object after the chunk size
//
DWORD CMaterial::GetRemainingSize()
{
	m_theHeader.dwChunkSize = GetFullSize () - (4 + sizeof (DWORD));
	return m_theHeader.dwChunkSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this object and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the material object, which includes
//	textures, to the output *.IM Indexed Mesh file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CMaterial::WriteToOutputFile(HANDLE	hOutputFile)
{
	CTexture *	ptrTexture;
	char		szName [MAX_PATH + 5];

	_strnset (szName, 0, GetNameSize (m_strDecoratedName));
	strcpy   (szName, m_strDecoratedName);
//
//			Write out the MATL header (with its fixed fields)
//
	m_theHeader.dwChunkSize = GetFullSize () - (4 + sizeof (DWORD));
	m_theHeader.dwNameSize = GetNameSize (m_strDecoratedName);

	if (! PerformTheWrite(hOutputFile, &m_theHeader, sizeof (m_theHeader), 
						  "Write of MATL Header failed"))
	{
		return false;
	}
//
//			Write out the name padded to the next DWORD boundary
//
	if (! PerformTheWrite(hOutputFile, &szName, GetNameSize (m_strDecoratedName), 
						  "Write of MATERIAL name failed"))
	{
		return false;
	}
//
//			Write out the remaining data values
//
	theAfterHeader.dwTextureCount = m_vTexture.size();
	if (! PerformTheWrite(hOutputFile, &theAfterHeader, sizeof (theAfterHeader), 
						  "Write of MATL After Header failed"))
	{
		return false;
	}
//
//			Write out each texture
//
	TEXTUREVECTOR::iterator pTexture = m_vTexture.begin ();
	while (pTexture != m_vTexture.end ())
	{
		ptrTexture = (*pTexture);
		ptrTexture->WriteToOutputFile (hOutputFile);
		pTexture++;
	}

	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CTexture object and put it into the texture vector
//
//   COMMENTS:
//
//	This method will create a new CTexture object and put it into the texture vector.
//	It will then return a pointer to that CTexture object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentTexture	Returned pointer to the new CTexture object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CMaterial::CreateNewTexture (CTexture*& pCurrentTexture)
{
//			Create the texture, insure that it is valid, and put it into the 
//			texture vector
//
	pCurrentTexture = new CTexture ();
	pCurrentTexture->AssertValid();
	m_vTexture.push_back (pCurrentTexture);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
