////////////////////////////////////////////////////////////////////////////////
//	Mesh.cpp:	The Jet Mesh object class
//
//	This class implements the Mesh object that describes the main components of a
//	3D model.  Meshes are made up of vertices that make up triangles, materials,
//	and textures.  The Mesh object contains the CHNK and the GEOM chunk chunks.
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Mesh.h"
#include "Float.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CMesh::CMesh()
{
	strncpy (&m_theCHNKHeader.chCHNKChunkID[0], "CHNK", 4);
	m_theCHNKHeader.dwVersion	 = 100;
	m_theCHNKHeader.dwChunkIndex = 0;

	strncpy (&m_theGEOMHeader.chGEOMChunkID[0], "GEOM", 4);
	m_theGEOMHeader;
	m_theGEOMHeader.dwVersion			= 102;
	m_theGEOMHeader.dwFlags				= 4;	// default to triangles.
	m_theGEOMHeader.fArea				= 1.0;	// default to 1.0, not used in Jet V1.0
	m_theGEOMHeader.dwVertexCount		= 0;
	m_theGEOMHeader.dwTriangleCount		= 0;
	m_theGEOMHeader.dwIndexCount		= 0;
	m_theGEOMHeader.dwFaceNormalCount	= 0;
	m_theGEOMHeader.dwMaxInfluence		= 0;
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CMesh::~CMesh()
{
//
//			Clean up the face objects pointed to by m_vFaces
//
	FACEVECTOR::iterator pFace = m_vFaces.begin ();
	while (pFace != m_vFaces.end ())
	{
		delete (CFace *) (*pFace);
		pFace++;
	}
	m_vFaces.clear();

//
//			Clean up the vertex objects pointed to by m_vVertices
//
	VERTEXVECTOR::iterator pVertex = m_vVertices.begin ();
	while (pVertex != m_vVertices.end ())
	{
		delete (CVertex *) (*pVertex);
		pVertex++;
	}
	m_vVertices.clear();

}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this CHNK object in the final .IM file
//
//   COMMENTS:
//
//	This method will add up the total lengths of all its internal components,
//	and return the total lengths.
//
//    RETURN VALUES:
//		The full size of this object in the .IM file
//
DWORD CMesh::GetFullSize ()
{
	DWORD	dwTotalSize		= 0;
//
//			Start with the start CHNK chunk
//
	dwTotalSize = sizeof (m_theCHNKHeader);				// header size
//
//			Add in the size of the Material chunk and all its textures.
//
	dwTotalSize += m_Material.GetFullSize ();	// size of the material chunk
//
//			Continue with the start of the GEOM chunk
//
	m_theGEOMHeader.dwVertexCount		= m_vVertices.size();
	m_theGEOMHeader.dwTriangleCount		= m_vFaces.size();
	m_theGEOMHeader.dwIndexCount		= m_vFaces.size() * 3;	// assume all triangles.
	m_theGEOMHeader.dwFaceNormalCount	= m_vFaces.size();		// assume one for each face

	m_dwGEOMFullSize =
			sizeof (m_theGEOMHeader) +					// GEOM header
			(m_theGEOMHeader.dwVertexCount   * 5 * sizeof (float)) +	// position & tex coords
			(m_theGEOMHeader.dwTriangleCount * 3 * sizeof (WORD))  +	// vertex indices
			(m_theGEOMHeader.dwVertexCount   * 3 * sizeof (float)) +	// vertex normals
			(m_theGEOMHeader.dwTriangleCount * 3 * sizeof (float));		// triangle indices
	dwTotalSize += m_dwGEOMFullSize;

	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the reamining size of this CHNK object after its chunk size
//
//   COMMENTS:
//
//	This method will get the total length of this object in the .IM file, and
//	will subtract the leading "CHNK" text and chunk size DWORD to return the
//	remaining size of the object.
//
//    RETURN VALUES:
//		The remaining size of this object after the chunk size
//
DWORD CMesh::GetRemainingSize ()
{
	m_theCHNKHeader.dwChunkSize = GetFullSize ()   - (4 + sizeof (DWORD));
	m_theGEOMHeader.dwChunkSize = m_dwGEOMFullSize - (4 + sizeof (DWORD));
	return m_theCHNKHeader.dwChunkSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this object and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the mesh object, which includes
//	materials (and associated textures), and geometry (which includes the the
//	mesh vertex and triangle information) to the output *.IM Indexed Mesh file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CMesh::WriteToOutputFile (HANDLE	hOutputFile)
{
//
//			Write out the CHNK header
//
	GetRemainingSize ();

	if (! PerformTheWrite(hOutputFile, &m_theCHNKHeader, sizeof (m_theCHNKHeader), 
						  "Write of CHNK Header failed"))
	{
		return false;
	}
//
//			Write out the material
//
	if (! m_Material.WriteToOutputFile (hOutputFile))
	{
		return false;
	}
//
//			Write out the GEOM header
//
	if (! PerformTheWrite(hOutputFile, &m_theGEOMHeader, sizeof (m_theGEOMHeader), 
						  "Write of GEOM Header failed"))
	{
		return false;
	}
//
//			Write out the vertex XYZ locations and texture UV coordinates
//
	CVertex *	ptrVertex;
	CFace *		ptrFace;

	VERTEXVECTOR::iterator pVertex = m_vVertices.begin ();
	while (pVertex != m_vVertices.end ())
	{
		ptrVertex = (*pVertex);
		if (! ptrVertex->WriteCoordsToOutputFile (hOutputFile))
		{
			return false;
		}
		pVertex++;
	}
//
//			Write out the vertices associated with each face
//
	FACEVECTOR::iterator pFace = m_vFaces.begin ();
	while (pFace != m_vFaces.end ())
	{
		ptrFace = (*pFace);
		if (! ptrFace->WriteVertsToOutputFile (hOutputFile))
		{
			return false;
		}
		pFace++;
	}
//
//			Write out the vertex normal values
//
	pVertex = m_vVertices.begin ();
	while (pVertex != m_vVertices.end ())
	{
		ptrVertex = (*pVertex);
		if (! ptrVertex->WriteNormalsToOutputFile (hOutputFile))
		{
			return false;
		}
		pVertex++;
	}
//
//			Write out the face normal values
//
	pFace = m_vFaces.begin ();
	while (pFace != m_vFaces.end ())
	{
		ptrFace = (*pFace);
		if (! ptrFace->WriteNormalsToOutputFile (hOutputFile))
		{
			return false;
		}
		pFace++;
	}

	return true;
}


////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CVertex object and put it into the vertex vector
//
//   COMMENTS:
//
//	This method will create a new CVertex object and put it into the vertex vector.
//	It will then return a pointer to that CVertex object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentVertex	Returned pointer to the new CVertex object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CMesh::CreateNewVertex(CVertex*& pCurrentVertex)
{
//			Create the vertex, insure that it is valid, and put it into the 
//			vertex vector
//
	pCurrentVertex = new CVertex ();
	pCurrentVertex->AssertValid();
	m_vVertices.push_back (pCurrentVertex);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CFace object and put it into the face vector
//
//   COMMENTS:
//
//	This method will create a new CFace object and put it into the face vector.
//	It will then return a pointer to that CFace object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentFace	Returned pointer to the new CFace object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CMesh::CreateNewFace(CFace*& pCurrentFace)
{
//			Create the face, insure that it is valid, and put it into the 
//			face vector
//
	pCurrentFace = new CFace ();
	pCurrentFace->AssertValid();
	m_vFaces.push_back (pCurrentFace);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: return the address of the texture vector
//
//   COMMENTS:
//
//	This method will return the address of the internal texture vector so that
//	external classes can do calculations on this vector.
//
//    RETURN VALUES:
//		The address of the internal texture vector.
//
VERTEXVECTOR* CMesh::GetVertexVector ()
{
	return &m_vVertices;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Calculate the face normal values
//
//   COMMENTS:
//
//	This method will calculate the face normal value for each face in the mesh.
//	Note that you may load the face normals directly, or calculate them here.
//
//    RETURN VALUES:
//		None.
//
void CMesh::CalculateFaceNormals()
{
	CFace *		ptrFace;

	FACEVECTOR::iterator pFace = m_vFaces.begin ();
	while (pFace != m_vFaces.end ())
	{
		ptrFace = (*pFace);
		ptrFace->CalculateNormal(m_vVertices[ptrFace->GetVertexIndex(0)],
								 m_vVertices[ptrFace->GetVertexIndex(1)],
								 m_vVertices[ptrFace->GetVertexIndex(2)]);
		pFace++;
	}
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Calculate the vertex normal values
//
//   COMMENTS:
//
//	This method will calculate the vertex normal value for each vertex in the mesh.
//	Note that you may load the vertex normals directly, or calculate them here.
//
//	Note also that you must calculate or set the face normals before you can have
//	this routine set the vertex normals, and you must have added the faces that
//	use each vertex to the m_anFaces array in each vertex.
//
//    RETURN VALUES:
//		None.
//
void CMesh::CalculateVertexNormals()
{
	CVertex *	ptrVertex;
	CFace *		ptrFace;
	float		fTotalx;
	float		fTotaly;
	float		fTotalz;
	float		fVNormalLength;

	VERTEXVECTOR::iterator pVertex = m_vVertices.begin ();
	while (pVertex != m_vVertices.end ())
	{
		ptrVertex = (*pVertex);
		fTotalx = 0.0;
		fTotaly = 0.0;
		fTotalz = 0.0;
//
//			If the vertex's face array index is zero, no faces use this vertex.
//			This should not happen, but if it does, set the Total X value
//			to something non-zero so we don't end up dividing by zero.
//
		int nVertexFaceIndex = ptrVertex->GetFaceIndex ();
		if (nVertexFaceIndex == 0)
		{
			fTotalx = 1.0;
		}
		else
		{
//			Otherwise, look up all the faces that include this index and
//			total the X, Y, and Z components of their face noormals.
//
			for (int nFaceIndex = 0; nFaceIndex < nVertexFaceIndex; nFaceIndex++)
			{
				ptrFace = m_vFaces[ptrVertex->GetFace (nFaceIndex)];
				fTotalx += ptrFace->GetNormal(0);
				fTotaly += ptrFace->GetNormal(1);
				fTotalz += ptrFace->GetNormal(2);
			}
//
//			Average the totals for the X, Y, and Z components of the surrounding
//			face normals to get the vertex "normal" vector
//
			fTotalx = fTotalx / nVertexFaceIndex;
			fTotaly = fTotaly / nVertexFaceIndex;
			fTotalz = fTotalz / nVertexFaceIndex;
//
//			Normalize the VNormals to make them a unit length
//
			fVNormalLength = (float)  sqrt ((fTotalx * fTotalx) + 
											(fTotaly * fTotaly) + 
											(fTotalz * fTotalz));

			fTotalx = fTotalx / fVNormalLength;
			fTotaly = fTotaly / fVNormalLength;
			fTotalz = fTotalz / fVNormalLength;
		}
//			Set the VNormal values for each vertex.
//
		ptrVertex->SetNormals (0, fTotalx);
		ptrVertex->SetNormals (1, fTotaly);
		ptrVertex->SetNormals (2, fTotalz);

		pVertex++;
	}
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Return the index of a vertex with the indicated position and mapping
//
//   COMMENTS:
//
//	This method will search through the vertex vector looking for an existing 
//	vertex with the indicated postion and texture mapping.  If found, it will return
//	the index of that vertex in the vector.  If not found, it will create a new
//	vertex with the indicated position and mapping, add it to the vector, and return
//	the index of the new vertex.
//
//	Note that this does not guarantee a truly unique vertex in the vector.  It is
//	possible to have multiple identical vertices in the vector before calling this
//	rouinte.  This logic will stop at the first existing vertex that matches the
//	inticated position and mapping without checking for multiple matches.
//
//    RETURN VALUES:
//		The index position of the indicated vertex in the vertex vector.
//
int CMesh::GetUniqueVertex(float x, float y, float z, float uv, float uw)
{
	CVertex *	ptrVertex;
	int			nIndex = 0;
//
//			Look for an existing vertex with the same postion and texture mapping
//
	VERTEXVECTOR::iterator pVertex = m_vVertices.begin ();
	while (pVertex != m_vVertices.end ())
	{
		ptrVertex = (*pVertex);
		if (
			   (ptrVertex->GetCoords   (0) == x)
			&& (ptrVertex->GetCoords   (1) == y)
			&& (ptrVertex->GetCoords   (2) == z)
			&& (ptrVertex->GetTexCoords(0) == uv)
			&& (ptrVertex->GetTexCoords(1) == uw)
		   )
		{
			return nIndex;
		}
		nIndex++;
		pVertex++;
	}
//
//			Not found, so create a new vertex from the indicated data and pass
//			back its position in the vector.
//
	CreateNewVertex (ptrVertex);
	ptrVertex->SetCoordinates (0, x);
	ptrVertex->SetCoordinates (1, y);
	ptrVertex->SetCoordinates (2, z);
	ptrVertex->SetTexCoords   (0, uv);
	ptrVertex->SetTexCoords   (1, uw);

	return m_vVertices.size() - 1;
}

////////////////////////////////////////////////////////////////////////////////
