////////////////////////////////////////////////////////////////////////////////
//	Mesh.h:	The Jet Mesh object class
//
//	This class defines the Mesh object that describes the main components of a
//	3D model.  Meshes are made up of vertices that make up triangles, materials,
//	and textures.  The Mesh object contains the CHNK and the GEOM chunk chunks.
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	NOTE: This is assumed to be an open-source project, so individual method
//	notes appear before each method in the CPP file rather than here in the
//	.h file.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MESH_H_INCLUDED_)
#define AFX_MESH_H_INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BaseIMObject.h"
#include "Vertex.h"
#include "Face.h"
#include "Material.h"
#include <vector>

typedef std::vector<CVertex *>	VERTEXVECTOR;
typedef std::vector<CFace *>	FACEVECTOR;

class CMesh : public CBaseIMObject  
{
public:
			CMesh	();
	virtual ~CMesh	();

	DWORD			GetFullSize				();
	DWORD			GetRemainingSize		();
	bool			WriteToOutputFile		(HANDLE		hOutputFile);
	bool			CreateNewVertex			(CVertex*&	pCurrentVertex);
	bool			CreateNewFace			(CFace*&	pCurrentFace);
	VERTEXVECTOR*	GetVertexVector			();
	void			CalculateFaceNormals	();
	void			CalculateVertexNormals	();
	int				GetUniqueVertex			(float		x,
											 float		y,
											 float		z,
											 float		uv,
											 float		uw);

	inline void SetMeshName					(char *		szMeshName)
	{
		strcpy (m_szMeshName, szMeshName);
	}

	inline char * GetMeshName				()
	{
		return m_szMeshName;
	}

	inline void SetChunkIndex				(DWORD		dwChunkIndex)
	{
		m_theCHNKHeader.dwChunkIndex = dwChunkIndex;
	}

	inline void SetMaxInfluence				(DWORD		dwMaxInfluence)
	{
		m_theGEOMHeader.dwMaxInfluence = dwMaxInfluence;
	}

	inline CMaterial * GetMaterial			()
	{
		return &m_Material;
	}

	inline CVertex * GetVertex				(int		nIndex)
	{
		if (m_vVertices.size() > nIndex)
			return m_vVertices[nIndex];
		else
			return NULL;
	}

	inline CFace * GetFace					(int		nIndex)
	{
		if (m_vFaces.size() > nIndex)
			return m_vFaces[nIndex];
		else
			return NULL;
	}

protected:

	typedef struct CHNK_Header
	{
		char	chCHNKChunkID [4];
		DWORD	dwChunkSize;
		DWORD	dwVersion;
		DWORD	dwChunkIndex;
	};
	CHNK_Header		m_theCHNKHeader;		// the mesh header values
//
//			Attributes
//
	VERTEXVECTOR	m_vVertices;			// the array of vertices for this mesh
	FACEVECTOR		m_vFaces;				// the array of faces for this mesh
	CMaterial		m_Material;				// the material for this mesh
	char			m_szMeshName[1024];		// the mesh's name

	typedef struct GEOM_Header
	{
		char	chGEOMChunkID [4];			// chunk ID
		DWORD	dwChunkSize;				// remaining chunk size
		DWORD	dwVersion;					// chunk version
		DWORD	dwFlags;					// general flags value
		float	fArea;						// approx area of geometry (not used in Jet V1.0)
		DWORD	dwVertexCount;				// total number of vertices
		DWORD	dwTriangleCount;			// total number of triangles
		DWORD	dwIndexCount;				// total number of triangle vertex indices
		DWORD	dwFaceNormalCount;			// total number of face normals
		DWORD	dwMaxInfluence;				// maximum number of influences
	};

	GEOM_Header		m_theGEOMHeader;		// the GEOM header data

	DWORD		m_dwGEOMFullSize;			// the full size of the current GEOM chunk
};

#endif // !defined(AFX_MESH_H_INCLUDED_)
////////////////////////////////////////////////////////////////////////////////
