////////////////////////////////////////////////////////////////////////////////
//	SkeletonBone.cpp:	Implements the Jet CSkeletonBone class
//
//	This class defines the recursive SkeletonBone that resides in an animation (kin) file.
//
//	Unlike most Jet "chunk" objects, this chunk is both a data storage object, and a
//	recursive container for zero, one, or more additional CSkeletonBone objects.
//	Note that most of the routines in this object will be recursive.
//
//	Like most of the Jet Objects, it is derived from CBaseIMObject, and implements
//	the virtual Get Size and Write to Output methods of that base class.
//
//	Copyright , Jack Emmerichs, 2001.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SkeletonBone.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

////////////////////////////////////////////////////////////////////////////////
// Construction/Destruction
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//	Default constructor
//
CSkeletonBone::CSkeletonBone()
{
	strncpy (&m_theHeader.chChunkID[0], "BONE", 4);
	m_theHeader.dwChunkSize	= 0;
	m_theHeader.dwVersion	= 100;
	m_theHeader.dwNameSize	= 0;
	m_dwChildBoneCount		= 0;
	m_szName[0]				= '\0';
}

////////////////////////////////////////////////////////////////////////////////
//	Standard destructor
//
CSkeletonBone::~CSkeletonBone()
{
//
//			Clean up the child bone objects pointed to by m_vChildBones
//
	BONEVECTOR::iterator pBone = m_vChildBones.begin ();
	while (pBone != m_vChildBones.end ())
	{
		delete (CSkeletonBone *) (*pBone);
		pBone++;
	}
	m_vChildBones.clear();
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the full size of this CSkeletonBone object in the final .KIN file
//
//   COMMENTS:
//
//	This method will return the length of the full CSkeletonBone heading..
//
//    RETURN VALUES:
//		The full size of this object in the .KIN file
//
DWORD CSkeletonBone::GetFullSize()
{
	CSkeletonBone *	ptrChildBone;
	DWORD			dwTotalSize	= sizeof (m_theHeader) +		// start with the header size
								  GetNameSize (m_szName) +		// add then name size
								  sizeof (m_dwChildBoneCount);	// plus last header value
//
//			Count the child bone objects we will use, and sum their total size.
//
	m_dwChildBoneCount = 0;
	BONEVECTOR::iterator pChildBone = m_vChildBones.begin ();
	while (pChildBone != m_vChildBones.end ())
	{
		ptrChildBone = (*pChildBone);
		dwTotalSize += ptrChildBone->GetFullSize ();
		m_dwChildBoneCount++;
		pChildBone++;
	}

	return dwTotalSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Get the reamining size of this INFO object after its chunk size
//
//   COMMENTS:
//
//	This method will get the total length of this object in the .KIN file, and
//	will subtract the leading "EVNT" text and chunk size DWORD to return the
//	remaining size of the object.
//
//    RETURN VALUES:
//		The remaining size of this object after the chunk size
//
DWORD CSkeletonBone::GetRemainingSize()
{
	m_theHeader.dwChunkSize = GetFullSize () - (4 + sizeof (DWORD));
	return m_theHeader.dwChunkSize;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Write this header and its contents to the output file
//
//   COMMENTS:
//
//	This method will write the contents of the INFO Header, which happens to be
//	the whole file, to the output *.KIN Animation file.
//
//	  hOutputFile		The handle of the output file to use
//
//    RETURN VALUES:
//		true:	If no errors occur
//		false:	Otherwise
//
bool CSkeletonBone::WriteToOutputFile(HANDLE			hOutputFile)
{
	CSkeletonBone *	ptrChildBone;
	char			szBoneName [MAX_PATH + 5];
//
//			Load the output variables with the proper values
//
	m_theHeader.dwNameSize = GetNameSize (m_szName);
	_strnset (szBoneName, 0, GetNameSize (m_szName));
	strcpy   (szBoneName, m_szName);
//
//			Write out the CSkeletonBone header
//
	GetRemainingSize();
	if (! PerformTheWrite(hOutputFile, &m_theHeader, sizeof (m_theHeader), 
						  "Write of CSkeletonBone Header failed"))
	{
		return false;
	}
//
//			Write out the name padded to the next DWORD boundary
//
	if (! PerformTheWrite(hOutputFile, &m_szName, GetNameSize (m_szName), 
						  "Write of INFO header mesh name failed"))
	{
		return false;
	}
//
//			Write out the child bone count
//
	if (! PerformTheWrite(hOutputFile, &m_dwChildBoneCount, sizeof (m_dwChildBoneCount), 
						  "Write of child bone count failed"))
	{
		return false;
	}
//
//			Write out each child bone object
//
	BONEVECTOR::iterator pChildBone = m_vChildBones.begin ();
	while (pChildBone != m_vChildBones.end ())
	{
		ptrChildBone = (*pChildBone);
		ptrChildBone->WriteToOutputFile (hOutputFile);
		pChildBone++;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
//   PURPOSE: Create a new CSkeletonBone object and put it into the ChildBones vector
//
//   COMMENTS:
//
//	This method will create a new CSkeletonBone object and put it into the ChildBones vector.
//	It will then return a pointer to that CSkeletonBone object so the calling routine can
//	do whatever it wants with it.
//
//	  pCurrentEventDetail		Returned pointer to the new CSkeletonBone object just created
//
//    RETURN VALUES:
//		true	If no errors occur
//		False	Otherwise
//
bool CSkeletonBone::CreateNewSkeletonBone (CSkeletonBone*& pCurrentSkeletonBone)
{
//			Create the CSkeletonBone, insure that it is valid, and put it into the 
//			ChildBones vector
//
	int	nChildBoneIndex = m_vChildBones.size();
	pCurrentSkeletonBone = new CSkeletonBone ();
	pCurrentSkeletonBone->AssertValid();
	m_vChildBones.push_back (pCurrentSkeletonBone);
	if (m_vChildBones.size() <= nChildBoneIndex)
		return false;
	else
		return true;
}

